; 
; Example for reading, computing, and displaying the time read by
; the monitor's subroutine $003B. By return from $003B the register DE
; contains the time in seconds and the accumulator register indicates
; AM or PM. If AM the the accu contains $00 otherwise it contains $01 for PM. 
;
; author Karl-Heinz Mau/Germany written for SHARP's MZ-700 system.

	CPU	Z80
	ORG	0B000H

CLS:	EQU	16H		;clear screen char
PRNT:   EQU	0012H		;subroutine to print 1 char from accu
BRKEY:	EQU	001EH		;checks control keys
TIMRD:	EQU	003BH		;reads sys time
ST1:	EQU	00ADH		;moni warm start
SPV:	EQU	10F0H		;stack top


; main routine

START:	LD      HL,0FFFFH	;init HL/force 1st time value computation process
        PUSH    HL		;save HL
START4:	LD      HL,WORK		;init workarea six ASCII chars "0"
        LD      A,'0'		;"0"
        LD      B,06H		;count = 6
LOOP1:  LD      (HL),A		;store "0" to workarea
        INC     HL		;next address in workarea
        DJNZ    LOOP1		;6 times
        POP     HL		;restore HL (contains last time read)
GETNEW:	CALL    TIMRD		;read time
        LD      B,A		;save AM/PM
        LD      A,D		;compare this time read and last read
        CP      H		;(I let display only different values..)
        JR      NZ,OK		;not equal, go
        LD      A,E
        CP      L
        JR      Z,GETNEW	;no difference, get new value
OK:	PUSH    DE		;becomes "old value" for next compare
        LD      HL,WORK		;points to hours most significant byte MSB
        LD      A,B		;restore AM/PM value
        OR      A
        JR      Z,LB029		;zero means AM (0 - 12)
        INC     (HL)		;+12 for PM (13 - 24)
        INC     HL		;my clock is a german one
        INC     (HL)		;it counts 24 hours
        INC     (HL)
        DEC     HL		;point to hours MSB
LB029:  EX      DE,HL		;to DE for subroutine COUNT
        LD      BC,36000	;10 hours = 36,000 seconds
        CALL    COUNT		;count number of ten hours
        LD      BC,3600		;1 hour = 3,600 seconds
        CALL    COUNT		;count number of hours
        CP      3AH		;overflow?
        JR      C,LB044		;no
        SUB     0AH		;yes, subtract 10 from hours lsb
        DEC     DE		;and correct both digits
        LD      (DE),A		;lsb
        DEC     DE		;msb
        LD      A,(DE)
        INC     A		;-1
        LD      (DE),A
        INC     DE
        INC     DE		;point to minutes msb
LB044:  DEC     DE
        DEC     DE		;hours msb
        LD      A,(DE)
        INC     DE		;hours lsb
        LD      B,A		;save hours msb to reg B
        LD      A,(DE)		;get hours lsb
        INC     DE		;point to minutes msb
        LD      C,A		;save hours lsb
        LD      A,B		;get hours msb
        CP      '2'		;check for 24 o' clock,
        JR      NZ,LB063	;no 
        LD      A,C		;if 24 then set time to 00:00:00
        CP      '4'		;because max value is 23:59:59
        JR      NZ,LB063	;not 24
        LD      HL,WORK		;it's 24, set to 000000
        LD      A,'0'
        LD      B,06H
LB05D:  LD      (HL),A
        INC     HL
        DJNZ    LB05D
        JR      LB079		;go print 00:00:00 o' clock

LB063:  LD      BC,600		;count number of 10 minutes msb
        CALL    COUNT
        LD      BC,60		;count number of minutes lsb
        CALL    COUNT
        LD      BC,10		;count number of 10 seconds msb
        CALL    COUNT
        LD      A,'0'
        ADD     A,L
        LD      (DE),A		;last digit, seconds
LB079:  LD      A,CLS
        CALL    PRNT		;clear screen
        LD      DE,WORK		;first digit of time
        LD      BC,0202H	;init loop counters
LB084:  LD      A,(DE)		;get digit (msb of hours or minutes)
        CALL    PRNT		;print digit
        INC     DE		;point to next digit
        LD      A,(DE)		;get this digit (lsb of hours or minutes)
        CALL    PRNT		;print 2nd digit
        LD      A,':'
        CALL    PRNT		;print ":"
        INC     DE		;point to next digit
        DJNZ    LB084		;loop 2 times until xx:xx: is printed
        LD      B,C		;setup second loop counter value
LB096:  LD      A,(DE)		;get digit (msb or lsb of seconds)
        CALL    PRNT		;print digit
        INC     DE		;point to next digit
        DJNZ    LB096		;loop 2 times until 2 digits printed
        CALL    BRKEY		;check for break condition, brk key?
        JP      NZ,START4	;no, proceed again
        LD      SP,SPV		;yes, restore stack to top
        JP      ST1		;goback to monitor


;  will subtract the value contained in register BC from register HL as
;  many times as possible. If the value contained in HL becomes negative then this
;  underrun is corected by a re-add.
;
;  on entry: HL = time in seconds
;	     BC = number of seconds for hours, minutes, etc.
;            DE = address of digit to count in workarea
;
;  on exit:  HL = decreased value by number of possible hours, minutes, etc. * BC
;            BC = unchanged
;	     DE = points to next digit now

COUNT:  OR      A		;reset carry flag
        SBC     HL,BC		;subtract BC from time value 
        JR      C,LB0B3		;negative value reached, so re-add
        LD      A,(DE)		;value + 1
        INC     A
        LD      (DE),A
        JR      COUNT		;try next count

LB0B3:  ADD     HL,BC		;adjust to positive value
        INC     DE		;point to next digit
        RET     		;goback to caller

WORK:	DB	"000000"
